@@ -134,14 +134,14 @@ display_about_dialog ()
         const gchar *authors[] =
                 {"Dodji Seketeli<dodji@mlview.org>",
                  "Gael Chamoulaud<strider@mlview.org>",
-                 "Nicolas Centa<happypeng@free.fr>"
-                 "",
-                NULL
+                 "Nicolas Centa<happypeng@free.fr>",
+                 "Baptiste Mille-Mathias<bmm80@ifrance.com>",
+                 "Stephane Bonhomme<s.bonhomme@wanadoo.fr>",
+                 NULL
         };
 
         const gchar *documenters[] =
                 {"Dodji Seketeli<dodji@mlview.org>",
-                 "Gael Chamoulaud <strider@mlview.org>",
                  NULL
         };
 
@@ -509,6 +509,21 @@ expand_tree_to_depth_cb (GtkWidget * a_menu_item,
         mlview_editor_execute_action (editor, &action) ;
 }
 
+static void
+apply_xslt_cb (GtkWidget * a_menu_item,
+                         MlViewAppContext * a_context)
+{
+        MlViewEditor *editor;
+
+        g_return_if_fail (a_context != NULL);
+        g_return_if_fail (a_menu_item != NULL);
+
+        editor = mlview_app_context_get_element (a_context,
+                                                 "MlViewEditor");
+        mlview_editor_xslt_transform_document_interactive 
+                (editor) ;
+}
+
 static void
 edit_settings_cb (GtkWidget * a_menu_item,
                   MlViewAppContext * a_context)
@@ -551,21 +566,11 @@ go_url_home_cb (void)
 
 /*the menuitems of the /file menu*/
 static GnomeUIInfo g_menu_files_tree[] = {
-        GNOMEUIINFO_MENU_OPEN_ITEM (open_cb, NULL),
-
-        GNOMEUIINFO_MENU_CLOSE_ITEM (close_cb, NULL),
-
         GNOMEUIINFO_MENU_NEW_ITEM (N_("New xml document"),
-                                   N_
-                                   ("Creates new xml document"),
+                                   N_("Creates new xml document"),
                                    new_cb, NULL),
 
-        GNOMEUIINFO_MENU_NEW_ITEM (N_
-                                   ("New view on xml document"),
-                                   N_
-                                   ("Creates a new view on current xml document"),
-                                   new_view_on_document_cb,
-                                   NULL),
+        GNOMEUIINFO_MENU_OPEN_ITEM (open_cb, NULL),
 
         GNOMEUIINFO_SEPARATOR,
 
@@ -573,12 +578,10 @@ static GnomeUIInfo g_menu_files_tree[] = {
 
         GNOMEUIINFO_MENU_SAVE_AS_ITEM (save_as_cb, NULL),
 
-        GNOMEUIINFO_ITEM (N_("Change view name"),
-                          N_("Rename the current view"),
-                          change_view_name_cb, NULL),
-
         GNOMEUIINFO_SEPARATOR,
 
+        GNOMEUIINFO_MENU_CLOSE_ITEM (close_cb, NULL),
+
         GNOMEUIINFO_MENU_EXIT_ITEM (exit_cb, NULL),
 
         GNOMEUIINFO_END
@@ -655,23 +658,38 @@ static GnomeUIInfo g_menu_edit_tree[] = {
 
 /*The */
 static GnomeUIInfo g_menu_actions_tree[] = {
+
+        GNOMEUIINFO_ITEM_STOCK (N_("New view on xml document"),
+                                N_("Creates a new view on current xml document"),
+                                new_view_on_document_cb, GTK_STOCK_NEW),
+
+        GNOMEUIINFO_ITEM (N_("Change view name"),
+                          N_("Rename the current view"),
+                          change_view_name_cb, NULL),
+
+        GNOMEUIINFO_SEPARATOR,
+
         GNOMEUIINFO_ITEM (N_("Associate a Dtd"),
-                          N_
-                          ("Associates a Dtd to the current xml document"),
+                          N_("Associates a Dtd to the current xml document"),
                           associate_a_dtd_cb, NULL),
 
         GNOMEUIINFO_ITEM (N_("Validate document"),
-                          N_
-                          ("Validates the document against it current Dtd"),
+                          N_("Validates the document against it current Dtd"),
                           validate_document_cb, NULL),
 
         GNOMEUIINFO_SEPARATOR,
 
         GNOMEUIINFO_ITEM (N_("Expand current node"),
-                          N_
-                          ("Expands the current selected node to an absolute depth"),
+                          N_("Expands the current selected node to an absolute depth"),
                           expand_tree_to_depth_cb, NULL),
 
+        GNOMEUIINFO_SEPARATOR,
+
+        GNOMEUIINFO_ITEM (N_("Apply XSLT..."),
+                          N_
+                          ("transform document using xslt stylesheet"),
+                          apply_xslt_cb, NULL),
+
         GNOMEUIINFO_END
 };
 
@@ -711,39 +729,27 @@ static GnomeUIInfo g_app_menu_bar[] = {
 
 /*the application toolbar*/
 static GnomeUIInfo g_toolbar[] = {
-        GNOMEUIINFO_ITEM_STOCK (N_("Open"),
-                                N_("open an existing file"),
-                                open_cb,
-                                GNOME_STOCK_PIXMAP_OPEN),
 
         GNOMEUIINFO_ITEM_STOCK (N_("New"),
                                 N_("creates a new xml document"),
                                 new_cb, GNOME_STOCK_PIXMAP_NEW),
 
+        GNOMEUIINFO_ITEM_STOCK (N_("Open"),
+                                N_("open an existing file"),
+                                open_cb,
+                                GNOME_STOCK_PIXMAP_OPEN),        
+
         GNOMEUIINFO_ITEM_STOCK (N_("Close"),
                                 N_
                                 ("closes the currently open file"),
                                 close_cb,
                                 GNOME_STOCK_PIXMAP_CLOSE),
 
-        GNOMEUIINFO_SEPARATOR,
-
         GNOMEUIINFO_ITEM_STOCK (N_("Save"),
                                 N_("save this file"),
                                 save_cb,
                                 GNOME_STOCK_PIXMAP_SAVE),
 
-        GNOMEUIINFO_ITEM_STOCK (N_("Save as"),
-                                N_
-                                ("save this file with a new name"),
-                                save_as_cb,
-                                GNOME_STOCK_PIXMAP_SAVE_AS),
-        GNOMEUIINFO_SEPARATOR,
-
-        GNOMEUIINFO_ITEM_STOCK (N_("Exit"),
-                                N_("Exits MlView application"),
-                                exit_cb,
-                                GNOME_STOCK_PIXMAP_QUIT),
         GNOMEUIINFO_END
 };
 
@@ -863,32 +869,6 @@ install_menu_and_toolbar (GnomeApp * a_appwin)
 }
 
 
-/**
- *Sets the icon of the current instance of GnomeApp to
- *the MlView's icon.
- *@param a_app instance of GnomeApp to consider.
- */
-void
-mlview_app_set_icon (GnomeApp * a_app)
-{
-        MlViewAppContext *app_context = NULL;
-
-        g_return_if_fail (a_app != NULL);
-        g_return_if_fail (GNOME_IS_APP (a_app));
-        g_return_if_fail (GTK_WIDGET (a_app)->window != NULL);
-
-        app_context = gtk_object_get_data (GTK_OBJECT (a_app),
-                                           "mlview-app-context");
-
-        g_return_if_fail (app_context != NULL);
-        g_return_if_fail (MLVIEW_IS_APP_CONTEXT (app_context));
-
-        mlview_app_context_set_window_icon (app_context,
-                                            GTK_WINDOW (a_app));
-}
-
-
-
 /**
  *Builds the mlview application.
  *@param a_appname the name of the mlview application.
@@ -898,16 +878,14 @@ GnomeApp *
 mlview_app_new (const guchar * a_appname)
 {
         GnomeApp *mlview_application = NULL;
-        GtkWidget *editor = NULL,
-                *appbar = NULL;
+        GtkWidget *editor = NULL;
+        GtkWidget *appbar = NULL;
+        gchar *icon_path = NULL;
 
-        mlview_application =
-                GNOME_APP (gnome_app_new (a_appname, a_appname));
+        mlview_application = GNOME_APP (gnome_app_new (a_appname, a_appname));
 
         /*creates the application context */
-        gv_app_ctxt =
-                MLVIEW_APP_CONTEXT
-                (mlview_app_context_get_instance ());
+        gv_app_ctxt = MLVIEW_APP_CONTEXT (mlview_app_context_get_instance ());
         gtk_object_set_data (GTK_OBJECT (mlview_application),
                              "mlview-app-context", gv_app_ctxt);
 
@@ -939,12 +917,28 @@ mlview_app_new (const guchar * a_appname)
         /*Install the menu and toolbar */
         install_menu_and_toolbar (mlview_application);
 
-        gtk_widget_show (editor);
-
         /*FIXME: the message printing is just for test purpose */
         mlview_app_context_sbar_set_default_message (gv_app_ctxt,
                                                      "MlView is ready");
 
+        /*set the icon, all the windows/dialogs inherit this icon */
+        icon_path = gnome_program_locate_file
+                (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
+                 "pixmaps/mlview-app-icon.png", TRUE,
+                 NULL);
+        if (icon_path) {
+                gtk_window_set_default_icon_from_file (icon_path, NULL);
+                g_free (icon_path);
+                icon_path = NULL;
+        } else {
+                mlview_utils_trace_info 
+                        ("could not load mlview-app-icon.png icon") ;
+        }
+
+        /*set the default window dimension */
+        gtk_window_set_default_size (GTK_WINDOW (mlview_application),
+                                     800, 600);
+
         /*
          *Make sure we exit the glib main loop when the user clicks the
          *window close button.
@@ -953,6 +947,8 @@ mlview_app_new (const guchar * a_appname)
                           "delete_event",
                           G_CALLBACK (delete_event_cb), gv_app_ctxt);
 
+        gtk_widget_show_all (GTK_WIDGET (mlview_application));
+
         return mlview_application;
 }
 
