@@ -60,8 +60,8 @@ enum MlViewIconTreeColumns  {
          */
         NB_COLUMNS
 };
-
-static guchar *node_to_string_tag (xmlNode * a_node);
+static guchar *node_to_string_tag (MlViewAppContext *a_ctxt,
+                                   xmlNode * a_node);
 static enum MlViewStatus build_tree_model_from_xml_tree (MlViewTreeEditor2 * a_this,
 							 const xmlNode * a_node,
 							 GtkTreeIter * a_ref_iter,
@@ -72,28 +72,41 @@ static void mlview_icon_tree_load_icons (MlViewIconTreeClass *a_klass);
 static enum MlViewStatus
 update_visual_node (MlViewTreeEditor2 *a_this, GtkTreeIter *a_iter)
 {
-        guchar *start_tag_str = NULL, *attr_str;
+        guchar *start_tag_str = NULL, *attr_str = NULL;
         GtkTreeModel *model = NULL;
         xmlNode *xml_node = NULL;
+        MlViewAppContext *ctxt = NULL ;
 
         g_return_val_if_fail (a_this
                               && MLVIEW_IS_TREE_EDITOR2 (a_this)
                               && a_iter, MLVIEW_BAD_PARAM_ERROR);
-  
+
+        ctxt = mlview_tree_editor2_get_application_context 
+                (a_this) ;
+        g_return_val_if_fail (ctxt, MLVIEW_BAD_PARAM_ERROR) ;
         model = mlview_tree_editor2_get_model (a_this);
         g_return_val_if_fail (model, MLVIEW_ERROR);
         gtk_tree_model_get (model, a_iter, XML_NODE_COLUMN,
                             &xml_node, -1);
   
-        start_tag_str = node_to_string_tag (xml_node);
-        attr_str = mlview_tree_editor2_build_attrs_list_str (xml_node);
+        start_tag_str = node_to_string_tag (ctxt, xml_node) ;
+        if (xml_node->type == XML_ELEMENT_NODE) {
+                attr_str = mlview_tree_editor2_build_attrs_list_str 
+                        (ctxt, xml_node);
+        }
         if (!start_tag_str)
                 return MLVIEW_OK ;
-  
-        gtk_tree_store_set (GTK_TREE_STORE (model),
-                            a_iter, NODE_COLUMN, 
-                            start_tag_str, ATTRIBUTES_COLUMN,
-                            attr_str, -1);
+
+        if (attr_str)
+                gtk_tree_store_set (GTK_TREE_STORE (model),
+                                    a_iter, NODE_COLUMN, 
+                                    start_tag_str, ATTRIBUTES_COLUMN,
+                                    attr_str, -1);
+        else
+                gtk_tree_store_set (GTK_TREE_STORE (model),
+                                    a_iter, NODE_COLUMN, 
+                                    start_tag_str, ATTRIBUTES_COLUMN,
+                                    "", -1);
 
         if (start_tag_str) {
                 g_free (start_tag_str) ;
@@ -119,6 +132,7 @@ build_tree_model_from_xml_doc (MlViewTreeEditor2 * a_this,
         GtkTreePath *tree_path = NULL;
         xmlNode *xml_tree = NULL;
         enum MlViewStatus status = MLVIEW_OK;
+        GHashTable *nodes_rows_hash = NULL ;
 
         g_return_val_if_fail (a_this
                               && MLVIEW_IS_TREE_EDITOR2 (a_this)
@@ -126,15 +140,21 @@ build_tree_model_from_xml_doc (MlViewTreeEditor2 * a_this,
                               && *a_model == NULL,
                               MLVIEW_BAD_PARAM_ERROR);
 
-        if (!a_this->nodes_rows_hash) {
-                a_this->nodes_rows_hash =
+        nodes_rows_hash = mlview_tree_editor2_get_nodes_rows_hash 
+                (MLVIEW_TREE_EDITOR2 (a_this));
+
+        if (!nodes_rows_hash) {
+                nodes_rows_hash =
                         g_hash_table_new (g_direct_hash,
                                           g_direct_equal);
-                if (!a_this->nodes_rows_hash) {
+                if (!nodes_rows_hash) {
                         mlview_utils_trace_info 
                                 ("The system may be out of memory");
                         return MLVIEW_ERROR;
                 }
+                mlview_tree_editor2_set_nodes_rows_hash
+                        (MLVIEW_TREE_EDITOR2 (a_this),
+                         nodes_rows_hash) ;
         }
         model = gtk_tree_store_new (NB_COLUMNS,
                                     G_TYPE_POINTER,
@@ -159,7 +179,7 @@ build_tree_model_from_xml_doc (MlViewTreeEditor2 * a_this,
                 mlview_utils_trace_info ("!row_ref failed") ;
                 goto cleanup ;
         }
-        g_hash_table_insert (a_this->nodes_rows_hash,
+        g_hash_table_insert (nodes_rows_hash,
                              (gpointer) a_doc, row_ref);
         gtk_tree_store_set (model, &iter, XML_NODE_COLUMN, a_doc, -1);
         gtk_tree_store_set (model, &iter, NODE_COLUMN,
@@ -199,31 +219,41 @@ build_tree_model_from_xml_tree (MlViewTreeEditor2 * a_this,
                 *parent_node = NULL;
         gchar *start_tag = NULL;
         enum MlViewStatus status = MLVIEW_OK;
+        GHashTable *nodes_rows_hash = NULL ;
+        MlViewAppContext *ctxt = NULL ;
 
         g_return_val_if_fail (a_this
                               && MLVIEW_IS_TREE_EDITOR2 (a_this)
                               && a_node && a_model && *a_model,
                               MLVIEW_BAD_PARAM_ERROR);
 
+        ctxt = mlview_tree_editor2_get_application_context 
+                (MLVIEW_TREE_EDITOR2 (a_this)) ;
+        g_return_val_if_fail (ctxt, MLVIEW_BAD_PARAM_ERROR) ;
+
         model = GTK_TREE_STORE (*a_model);
         g_return_val_if_fail (model, MLVIEW_BAD_PARAM_ERROR);
-
-        if (!a_this->nodes_rows_hash) {
-                a_this->nodes_rows_hash =
+        nodes_rows_hash = mlview_tree_editor2_get_nodes_rows_hash 
+                (MLVIEW_TREE_EDITOR2 (a_this)) ;        
+        if (!nodes_rows_hash) {
+                nodes_rows_hash =
                         g_hash_table_new (g_direct_hash,
                                           g_direct_equal);
-                if (!a_this->nodes_rows_hash) {
+                if (!nodes_rows_hash) {
                         mlview_utils_trace_info
                                 ("The system may be out of memory");
                         return MLVIEW_ERROR;
                 }
+                mlview_tree_editor2_set_nodes_rows_hash 
+                        (MLVIEW_TREE_EDITOR2 (a_this),
+                         nodes_rows_hash) ;
         }
-
+        
         for (cur_node = (xmlNode *) a_node;
              cur_node; cur_node = cur_node->next) {
                 GtkTreeRowReference *row_ref = NULL;
 
-                start_tag = node_to_string_tag (cur_node);
+                start_tag = node_to_string_tag (ctxt, cur_node);
                 switch (a_type) {
                 case INSERT_TYPE_ADD_CHILD:
                         gtk_tree_store_append (model, &iter,
@@ -286,7 +316,7 @@ build_tree_model_from_xml_tree (MlViewTreeEditor2 * a_this,
                         goto cleanup ;
                 }
                 g_hash_table_insert
-                        (a_this->nodes_rows_hash,
+                        (nodes_rows_hash,
                          cur_node, row_ref);
                 gtk_tree_store_set (model, &iter,
                                     XML_NODE_COLUMN, cur_node, -1);
@@ -302,7 +332,8 @@ build_tree_model_from_xml_tree (MlViewTreeEditor2 * a_this,
                                             MLVIEW_ICON_TREE_CLASS
                                             (G_OBJECT_GET_CLASS (a_this))->icons.open_element,
                                             ATTRIBUTES_COLUMN,
-                                            mlview_tree_editor2_build_attrs_list_str (cur_node),
+                                            mlview_tree_editor2_build_attrs_list_str 
+                                            (ctxt, cur_node),
                                             ARE_ATTRIBUTES_EDITABLE_COLUMN,
                                             TRUE,
                                             -1);
@@ -327,7 +358,7 @@ build_tree_model_from_xml_tree (MlViewTreeEditor2 * a_this,
                 } else if (cur_node->type == XML_COMMENT_NODE) {
                         gtk_tree_store_set (model, &iter, 
                                             IS_EDITABLE_COLUMN,
-                                            FALSE,
+                                            TRUE,
                                             OPEN_ICON_COLUMN,
                                             MLVIEW_ICON_TREE_CLASS
                                             (G_OBJECT_GET_CLASS (a_this))->icons.comment,
@@ -335,17 +366,41 @@ build_tree_model_from_xml_tree (MlViewTreeEditor2 * a_this,
                                             FALSE,
                                             -1);
 		} else if (cur_node->type == XML_PI_NODE) {
-                        gtk_tree_store_set (model, &iter, 
+                        gtk_tree_store_set (model, &iter,
                                             IS_EDITABLE_COLUMN,
                                             FALSE,
                                             OPEN_ICON_COLUMN,
                                             MLVIEW_ICON_TREE_CLASS
-                                            (G_OBJECT_GET_CLASS (a_this))->icons.pi,
+                                            (G_OBJECT_GET_CLASS 
+                                             (a_this))->icons.pi,
                                             ARE_ATTRIBUTES_EDITABLE_COLUMN,
                                             FALSE,
                                             -1);
-                } else {
-                        mlview_utils_trace_info ("unknown node");
+                } else if (cur_node->type == XML_DTD_NODE) {
+                        gtk_tree_store_set (model, &iter,
+                                            IS_EDITABLE_COLUMN,
+                                            FALSE,
+                                            ARE_ATTRIBUTES_EDITABLE_COLUMN,
+                                            FALSE,
+                                            -1);
+                        if (cur_node->children) {
+                                build_tree_model_from_xml_tree
+                                        (a_this,
+                                         cur_node->children,
+                                         &iter,
+                                         INSERT_TYPE_ADD_CHILD,
+                                         a_model);
+                        }
+                } else if (cur_node->type == XML_ENTITY_DECL) {
+                        gtk_tree_store_set
+                                (model, &iter,
+                                 IS_EDITABLE_COLUMN, FALSE,
+                                 ARE_ATTRIBUTES_EDITABLE_COLUMN,
+                                 FALSE,
+                                 -1);
+                }else {
+                        mlview_utils_trace_info 
+                                ("unknown type of node");
                 }
                 if (start_tag) {
                         g_free (start_tag) ;
@@ -382,16 +437,25 @@ build_tree_model_from_xml_tree (MlViewTreeEditor2 * a_this,
         return status ;
 }
 
+/**
+ *Builds a start tag string 
+ *(e.g: <node-name attr0="val0" attr1="val1">)out of
+ *an instance of xmlNode *
+ *@param a_node the instance of xmlNode * to consider.
+ *@return the newly built start tag string.
+ */
 static guchar *
-node_to_string_tag (xmlNode * a_node)
+node_to_string_tag (MlViewAppContext *a_ctxt,
+                    xmlNode * a_node)
 {
         guchar *result = NULL,
-                *content = NULL;
-  
-        g_return_val_if_fail (a_node != NULL, NULL);
-  
-        const gchar *colour_str = mlview_tree_editor2_get_colour_string (a_node->type);
-  
+                *content = NULL,
+                *escaped_content = NULL;
+        const gchar *colour_str = 
+                mlview_tree_editor2_get_colour_string 
+                (a_ctxt, a_node->type);
+
+        g_return_val_if_fail (a_node != NULL, NULL) ;
         if (a_node->type == XML_ELEMENT_NODE) {
                 guchar *ns_prefix = NULL,
                         *name = NULL;    
@@ -410,17 +474,16 @@ node_to_string_tag (xmlNode * a_node)
                         g_free (ns_prefix);
                         ns_prefix = NULL;
                 }
-    
-                result = g_strdup_printf ("<span foreground=\"%s\">%s</span>", 
+
+                result = g_strdup_printf ("<span foreground=\"%s\">%s</span>",
                                           colour_str, name);
-    
                 if (name) {
                         g_free (name);
                         name = NULL;
                 }
     
         } else if (xmlNodeIsText (a_node)) {
-                guchar *escaped_content = NULL, *str = NULL;
+                guchar *str = NULL;
                 guint esc_content_len = 0 ;
                 enum MlViewStatus status = MLVIEW_OK ;
                 content = xmlNodeGetContent (a_node);
@@ -456,23 +519,183 @@ node_to_string_tag (xmlNode * a_node)
                         xmlNodeSetContent (a_node, "<!--comment-->");
                         content = xmlNodeGetContent (a_node); }
     
-                result = g_strdup_printf ("<span foreground=\"%s\">%s</span>",
-                                          colour_str, content);
-    
+                escaped_content = g_markup_escape_text (content,
+                                                        strlen (content));
+                if (!escaped_content) {
+                        mlview_utils_trace_info 
+                                ("g_markup_escape_text") ;
+                        xmlFree (content) ;
+                        return NULL ;
+                }
+                result = g_strdup_printf 
+                        ("<span foreground=\"%s\">%s</span>",
+                         colour_str, escaped_content);
                 xmlFree (content);
+                content = NULL ;
+                g_free (escaped_content) ;
+                
         } else if (a_node->type == XML_PI_NODE) {
                 content = xmlNodeGetContent (a_node);
                 if (content == NULL) {
                         xmlNodeSetContent
                                 (a_node, "processing instruction node");
-                        content = xmlNodeGetContent (a_node); }
-    
-                result = g_strdup_printf ("<span foreground=\"%s\">%s %s</span>",
-                                          colour_str, a_node->name, content);
+                        content = xmlNodeGetContent (a_node);
+                        if (!content) {
+                                mlview_utils_trace_info 
+                                        ("xmlNodeGetContent() failed") ;
+                                return NULL ;                                
+                        }
+                }   
+                escaped_content = g_markup_escape_text
+                        (content, strlen (content)) ;
+                if (!escaped_content) {
+                        mlview_utils_trace_info
+                                ("g_markup_escape_text() failed") ;
+                        xmlFree (content) ;
+                        return NULL ;
+                }
+                result = g_strdup_printf 
+                        ("<span foreground=\"%s\">%s %s</span>",
+                         colour_str, a_node->name, escaped_content);
+                if (content) {
+                        xmlFree (content);
+                        content = NULL ;
+                }
+                if (escaped_content) {
+                        g_free (escaped_content) ;
+                        escaped_content = NULL ;
+                }
+        } else if (a_node->type == XML_DTD_NODE) {
+                xmlDtd *node = (xmlDtd*)a_node ;
+                guchar *tmp_str = NULL ;
+                guchar *dtd_color = (guchar*)
+                        mlview_tree_editor2_get_colour_string
+                        (a_ctxt, a_node->type) ;
+                if (!a_node->name) {
+                        mlview_utils_trace_info 
+                                ("a node of type XML_DTD_NODE must have a ->name field set!!") ;
+                        return NULL ;
+                }
+                content = g_strconcat 
+                        ("<span foreground=\"",
+                         dtd_color, "\">"
+                         "&lt;!DOCTYPE ",
+                         a_node->name, NULL) ;
+                if (!content) {
+                        mlview_utils_trace_info 
+                                ("g_strconcat failed") ;
+                        return NULL ;
+                }
+                if (node->ExternalID) {
+                        escaped_content =
+                                g_markup_escape_text
+                                (node->ExternalID, strlen (node->ExternalID)) ;
+                        if (!escaped_content) {
+                                mlview_utils_trace_info
+                                        ("g_markup_escaped_text() failed") ;
+                                return NULL ;
+                        }
+                        tmp_str = g_strconcat (content, 
+                                               " PUBLIC \"", 
+                                               escaped_content,
+                                               "\"",
+                                               NULL) ;
+                        if (escaped_content) {
+                                g_free (escaped_content) ;
+                                escaped_content = NULL;
+                        }
+                        if (content) {
+                                g_free (content) ;
+                                content = NULL ;
+                        }
+                        content = tmp_str ; tmp_str = NULL ;
+                        if (node->SystemID) {
+                                escaped_content = 
+                                        g_markup_escape_text 
+                                        (node->SystemID, 
+                                         strlen (node->ExternalID)) ;
+                                if (!escaped_content) {
+                                        mlview_utils_trace_info 
+                                                ("g_markup_escape_text() failed") ;
+                                        return NULL ;
+                                }
+                                tmp_str = g_strconcat 
+                                        (content,
+                                         " \"",
+                                         escaped_content,
+                                         "\"&gt;</span>",
+                                         NULL) ;
+                                g_free (escaped_content) ;
+                                if (!tmp_str) {
+                                        mlview_utils_trace_info
+                                                ("g_strconcat failed.") ;
+                                        return NULL ;
+                                }
+                                g_free (content) ;
+                                content = tmp_str ; tmp_str = NULL ;
+                        }
+                } else {
+                        if (node->SystemID) {
+                                escaped_content = g_markup_escape_text 
+                                        (node->SystemID, 
+                                         strlen (node->SystemID)) ;
+                                if (!escaped_content) {
+                                        mlview_utils_trace_info 
+                                                ("g_markup_escape_text failed") ;
+                                        return NULL ;
+                                }
+                                tmp_str = g_strconcat 
+                                        (content,
+                                         " SYSTEM \"",
+                                         escaped_content,
+                                         "\"&gt;</span>",
+                                         NULL) ;
+                                g_free (escaped_content) ;
+                                if (!tmp_str) {
+                                        mlview_utils_trace_info 
+                                                ("g_strconcat failed") ;
+                                        return NULL ;
+                                }
+                                content = tmp_str ; tmp_str = NULL ;
+                        }
+                }
+                result = content ;
+                content = NULL ;
+        } else if (a_node->type == XML_ENTITY_DECL) {
+                xmlEntity *entity = (xmlEntity*) a_node ;
+                switch (entity->etype) {
+                case XML_INTERNAL_GENERAL_ENTITY:
+                        mlview_tree_editor2_internal_general_entity_to_string
+                                (a_ctxt, entity, &result) ;
+                        break ;
+                case XML_EXTERNAL_GENERAL_PARSED_ENTITY:
+                        mlview_tree_editor2_external_general_parsed_entity_to_string 
+                                (a_ctxt, entity, &result) ;
+                        break ;
+                case XML_EXTERNAL_GENERAL_UNPARSED_ENTITY:
+                        mlview_tree_editor2_external_general_unparsed_entity_to_string
+                                (a_ctxt, entity, &result) ;
+                        break ;
+                case XML_INTERNAL_PARAMETER_ENTITY:
+                        mlview_tree_editor2_internal_parameter_entity_to_string
+                                (a_ctxt, entity, &result) ;
+                        break ;
+                case XML_EXTERNAL_PARAMETER_ENTITY:
+                        mlview_tree_editor2_external_parameter_entity_to_string
+                                (a_ctxt, entity, &result) ;
+                        break ;
 
-                xmlFree (content);
-        }
-  
+                case XML_INTERNAL_PREDEFINED_ENTITY:
+                        mlview_utils_trace_info
+                                ("Oops, dunno how to render "
+                                 "XML_INTERNAL_PREDEFINED_ENTITY "
+                                 "type of xml entity decl node") ;
+                        break ;
+                default:
+                        mlview_utils_trace_info 
+                                ("Unknown entity type") ;
+                }
+        }  
         return result;
 }
 
@@ -594,12 +817,14 @@ node_to_xml_tag_w_attr (const xmlChar *node_name, xmlElementType type, xmlNode *
 }
 
 static guchar *
-node_to_xml_tag (const xmlChar *node_name, xmlElementType type, xmlNode *children, xmlNs *ns, 
+node_to_xml_tag (const xmlChar *node_name,
+                 xmlElementType type, xmlNode *children, xmlNs *ns, 
 		 xmlChar *content, xmlAttrPtr attributes)
 {
-        guchar *attr, *result;
+        guchar *attr=NULL, *result=NULL;
 
-        attr = build_xml_attrs_list_str (attributes);
+        if (attributes && attributes->name)
+                attr = build_xml_attrs_list_str (attributes);
         result = node_to_xml_tag_w_attr (node_name, type, children, ns, content, attr);
   
         if (attr)
@@ -609,12 +834,14 @@ node_to_xml_tag (const xmlChar *node_name, xmlElementType type, xmlNode *childre
 }
 
 static guchar *
-build_attrs_list_str_from_nv_pairs (GList *list)
+build_attrs_list_str_from_nv_pairs (MlViewAppContext *a_ctxt,
+                                    GList *list)
 {
         guchar *result = NULL, *tmp;
         struct NameValuePair *pair;
         const gchar *attval_col = "#00FF00",
-                *attname_col = mlview_tree_editor2_get_colour_string (XML_ATTRIBUTE_NODE);
+                *attname_col = mlview_tree_editor2_get_colour_string 
+                (a_ctxt, XML_ATTRIBUTE_NODE);
 
         while (list)
         {
@@ -749,7 +976,8 @@ node_cell_edited_cb (GtkCellRendererText *a_renderer,
                 mlview_utils_trace_info ("cur_node failed") ;
                 goto cleanup ;
         }
-        mlview_xml_doc = mlview_tree_editor2_get_mlview_xml_doc (tree_editor);
+        mlview_xml_doc = 
+                mlview_tree_editor2_get_mlview_xml_doc (tree_editor);
         if (!mlview_xml_doc) {
                 mlview_utils_trace_info 
                         ("mlview_xml_doc failed") ;
@@ -807,24 +1035,23 @@ build_tree_view_from_xml_doc (MlViewTreeEditor2 * a_this,
         GtkTreeModel *model = NULL ;
         GtkCellRenderer *renderer = NULL ;
         GtkTreeIter iter = {0} ;
+        GtkTreeViewColumn *column = NULL ;
         enum MlViewStatus status = MLVIEW_OK ;
         struct MlViewAppSettings *settings = NULL ;
+        MlViewAppContext *ctxt = NULL ;
         gboolean is_ok = TRUE ;
-	GtkTreeViewColumn *column;
 
         g_return_val_if_fail (a_this
-                              && MLVIEW_IS_TREE_EDITOR2 (a_this)
-                              && a_this->app_context, 
+                              && MLVIEW_IS_TREE_EDITOR2 (a_this),
                               NULL);
 
-        settings = mlview_app_context_get_settings 
-                (a_this->app_context) ;
-	
+        ctxt = mlview_tree_editor2_get_application_context 
+                (a_this) ;
+        g_return_val_if_fail (ctxt, NULL) ;
+        settings = mlview_app_context_get_settings (ctxt) ;
         g_return_val_if_fail (settings, NULL) ;
-
         status = build_tree_model_from_xml_doc
-                (a_this, a_doc, &model);
-	
+                (a_this, a_doc, &model);	
         g_return_val_if_fail (model, NULL);
 
         is_ok = gtk_tree_model_get_iter_first (model, &iter) ;
@@ -859,15 +1086,20 @@ build_tree_view_from_xml_doc (MlViewTreeEditor2 * a_this,
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_insert_column_with_attributes
                 (tree_view, 1, _("Attributes"),
-                 renderer, "markup", ATTRIBUTES_COLUMN, "editable", 
+                 renderer, "markup", ATTRIBUTES_COLUMN, "editable",
                  ARE_ATTRIBUTES_EDITABLE_COLUMN, NULL);
+
+        if (column) {
+                gtk_tree_view_column_set_resizable 
+                        (column, TRUE) ;
+        }
 	g_signal_connect (G_OBJECT (renderer),
 			  "edited", G_CALLBACK (node_attributes_edited_cb),
 			  a_this);
 	
         mlview_utils_gtk_tree_view_expand_row_to_depth2
                 (tree_view, &iter,
-                 settings->default_doc_expansion_depth) ;
+                 settings->tree_editors.default_doc_expansion_depth) ;
 
         return tree_view;
 }
