@@ -20,9 +20,7 @@
  *If not, write to the Free Software Foundation, 
  *Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *
- *Copyright 2001-2002 dodji SEKETELI, Gal CHAMOULAUD.
- *http://www.freespiders.org
+ *See COPYRIGHT file for copyright information.
  */
 
 #ifndef __MLVIEW_PREFERENCES_H__
@@ -32,9 +30,15 @@
 #include "mlview-preferences.h"
 #include "mlview-app-context.h"
 #include "config.h"
-
-void
- mlview_preferences_dialog_show (MlViewAppContext * a_context);
-
+/**
+ *@file
+ *declaration of the preferences editor gui.
+ *This is work in progress, does not work
+ *and is maybe going to be rewriten in a near
+ *future.
+ */
+G_BEGIN_DECLS
+void mlview_preferences_dialog_show (MlViewAppContext * a_context);
+G_END_DECLS
 
 #endif
