@@ -487,42 +487,34 @@ mlview_view_adapter_disconnect_from_document (MlViewViewAdapter * a_this,
 static GtkWidget *
 build_name_edition_dialog (MlViewAppContext * a_app_context)
 {
-        GtkDialog *result = NULL;
-        GtkEntry *name_entry_field = NULL;
+        GtkWidget *result = NULL;
+        GtkWidget *name_entry_field = NULL;
 
         g_return_val_if_fail (a_app_context != NULL, NULL);
         g_return_val_if_fail (MLVIEW_IS_APP_CONTEXT
                               (a_app_context), NULL);
 
-        result = GTK_DIALOG
-                (gtk_dialog_new_with_buttons
+        result = gtk_dialog_new_with_buttons
                  (_("Type the name of the current view"), NULL,
                   GTK_DIALOG_MODAL, 
                   GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, 
                   GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,                  
-                  NULL));
-        g_return_val_if_fail (result, NULL) ;
+                  NULL);
         gtk_dialog_set_default_response (GTK_DIALOG (result),
                                          GTK_RESPONSE_ACCEPT) ;
-        name_entry_field = GTK_ENTRY (gtk_entry_new ());
 
-        gtk_widget_show_all (GTK_WIDGET (name_entry_field));
+        name_entry_field = gtk_entry_new ();
 
         gtk_object_set_data (GTK_OBJECT (result),
                              NAME_EDITION_ENTRY_FIELD_KEY,
                              name_entry_field);
+        gtk_box_pack_start_defaults (GTK_BOX 
+                                     (GTK_DIALOG (result)->vbox),
+                                     name_entry_field) ;
 
-        gtk_box_pack_start_defaults (GTK_BOX (result->vbox),
-                                     GTK_WIDGET
-                                     (name_entry_field));
+        gtk_widget_show_all (result);
 
-        gtk_widget_realize (GTK_WIDGET (result));
-        mlview_app_context_set_window_icon (a_app_context,
-                                            GTK_WINDOW (result));
-
-        gtk_widget_show_all (GTK_WIDGET (result->vbox));
-
-        return GTK_WIDGET (result);
+        return result;
 }
 
 
